"""
This script will test the scaling and performance of the code on your system.

It requires some user input, which you can define at the top of the script after the imports.

The following values should be configured according to your system:
-

It will then run the population you specified, first linearly <amt_repeats> times,
and then using multiprocessing it will run the population <amt_repeats> times each time
with more cores. (Up until <amount_of_cpus>)

TODO: get the real evolution time instead of the total as well
TODO: put the methods in functions and put them in a different file
"""

import os
import json
import socket
import psutil

import numpy as np

from binarycpython.utils.grid import Population
from scaling_functions import (
    get_mp_results,
    get_linear_results,
    run_systems_for_scaling_comparison,
)
from plot_scaling import plot_speedup_and_efficiency

settings_dict = {}
settings_dict[
    "amt_repeats"
] = 1  # Number of times the population will be repeated per cpu
# number. Better do it several times than only run it once
settings_dict[
    "resolutions"
] = [  # List of resolution of sampling of the population. Useful for checking whether population size has an effect on the results
    {"M_1": 10, "per": 10, "q": 2}
]
settings_dict[
    "result_dir"
] = "scaling_results"  # Relative of absolute directory where results are writting to
settings_dict["plot_dir"] = "scaling_plots"  # Directory where the plots will be stored
settings_dict[
    "testcase"
] = "linear vs MP batched"  # 'name' of the calculation. will be used in the plot
settings_dict[
    "stepsize_cpus"
] = 1  # Stepsize for the cpu number generator. Try to keep this
# low, to get the most reliable results
settings_dict[
    "amount_of_cpus"
] = 4  # Amount of logical cpus the machine has (this is not the same as physical cpus!)
# settings_dict['amount_of_cpus'] = psutil.cpu_count()
settings_dict["amount_of_cores"] = 2  # The amount of physical cores. This value
# is not vital bit will be used in the plot
# settings_dict['amount_of_cores'] = psutil.cpu_count(logical=False)   # You can also use the psutil function to get
# the amt of physical cores, but this isnt fully
# reliable (in mar 2020 it didnt get this value
# right when there were multiple sockets)

run_systems_for_scaling_comparison(settings_dict)
#################################
# Files
SCALING_RESULT_DIR = settings_dict["result_dir"]
RESULT_JSONS = [
    os.path.join(SCALING_RESULT_DIR, file)
    for file in os.listdir(SCALING_RESULT_DIR)
    if file.endswith(".json")
]  # Automatically grab all of the stuff, override it

# FILENAMES = [
#     "david-Lenovo-IdeaPad-S340-14IWL_100_systems.json",
#     "david-Lenovo-IdeaPad-S340-14IWL_2500_systems.json"
# ]
# RESULT_JSONS = []
# for filename in FILENAMES:
#     RESULT_JSONS.append(os.path.join(os.path.abspath(SCALING_RESULT_DIR), filename))

plot_speedup_and_efficiency(RESULT_JSONS, SCALING_RESULT_DIR, "Example")
